from fastapi import Depends, Response
import jinja2

from chlore.request import get_request


class JinjaResponse(Response):
    media_type = "text/html"

    def __init__(
        self,
        template: jinja2.Template,
        context: dict,
        status_code: int = 200,
        headers: dict = None,
        media_type: str = None,
        background=None,
    ):
        super().__init__(template.render(context), status_code, headers, media_type, background)


def _build_jinja_env(loader: jinja2.BaseLoader) -> jinja2.Environment:
    def url_for(name: str, **path_params) -> str:
        return get_request().url_for(name, **path_params)

    def static(name: str) -> str:
        return url_for("static", path=name)

    env = jinja2.Environment(loader=loader, autoescape=True)
    env.globals["url_for"] = url_for
    env.globals["static"] = static
    return env


class JinjaTemplateLookupError(Exception):
    pass


class JinjaResponseBuilder:
    def __init__(self, loader: jinja2.BaseLoader):
        self.env = _build_jinja_env(loader)

    @classmethod
    def with_root(cls, root: str) -> "JinjaResponseBuilder":
        def load_template(name: str) -> str:
            if ":" in name:
                namespace, name = name.split(":", maxsplit=1)
                path = f"{namespace}/templates/{name}"
            else:
                path = f"templates/{name}"
            try:
                with open(f"{root}/{path}", "r") as f:
                    return f.read()
            except OSError:
                raise JinjaTemplateLookupError(path)

        return cls(jinja2.FunctionLoader(load_template))

    def __call__(
        self,
        name: str,
        context: dict,
        status_code: int = 200,
        headers: dict = None,
        media_type: str = None,
        background=None,
    ) -> JinjaResponse:
        return JinjaResponse(self.env.get_template(name), context, status_code, headers, media_type, background)


def jinja_mixin_with_response_builder(builder: JinjaResponseBuilder):
    """
    Create a controller mixin class with a JinjaResponseBuilder dependency

    :param builder:    the response builder to use
    """

    class _JinjaMixin:
        render: JinjaResponseBuilder = Depends(lambda: builder)

    return _JinjaMixin



import logging.config

import structlog

from .config import LoggingConfig
from .request import get_request


def request_logger() -> structlog.stdlib.BoundLogger:
    """
    Retrieve the logger associated with the current request
    """
    return structlog.get_logger().bind(request_id=get_request().state.id)


def get_standard_logging_config(config: LoggingConfig):
    processors = [
        structlog.stdlib.add_log_level,
        structlog.stdlib.add_logger_name,
        structlog.processors.TimeStamper(fmt="iso"),
    ]

    logging_config = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "json": {
                "()": structlog.stdlib.ProcessorFormatter,
                "processor": structlog.processors.JSONRenderer(),
                "foreign_pre_chain": processors,
            },
            "console": {
                "()": structlog.stdlib.ProcessorFormatter,
                "processor": structlog.dev.ConsoleRenderer(),
                "foreign_pre_chain": processors,
            },
        },
        "handlers": {
            "default": {
                "level": "DEBUG",
                "class": "logging.StreamHandler",
                "formatter": config.format,
            },
        },
        "loggers": {
            "": {
                "handlers": ["default"],
                "level": "INFO",
            },
            "uvicorn": {
                "handlers": ["default"],
                "level": "INFO",
                "propagate": False,
            },
            "uvicorn.error": {
                "handlers": ["default"],
                "level": "INFO",
                "propagate": False,
            },
            "uvicorn.access": {
                "handlers": ["default"],
                "level": "INFO",
                "propagate": False,
            },
            "pika": {
                "handlers": ["default"],
                "level": "WARNING",
                "propagate": False,
            },
        },
    }
    return logging_config


def setup_standard_logging(config: LoggingConfig):
    logging.config.dictConfig(get_standard_logging_config(config))


def setup_app_logging(config: LoggingConfig):
    processors = [
        structlog.processors.add_log_level,
        structlog.processors.StackInfoRenderer(),
        structlog.dev.set_exc_info,
        structlog.processors.format_exc_info,
        structlog.processors.TimeStamper(fmt="ISO"),
    ]

    if config.format == "json":
        processors.append(structlog.processors.JSONRenderer())
    else:  # config.format == "text"
        processors.append(structlog.dev.ConsoleRenderer())

    structlog.configure(processors=processors)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chlore']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'SQLAlchemy',
 'fastapi>=0.79.0,<0.80.0',
 'pydantic>=1.9.1,<2.0.0',
 'structlog>=21.5.0,<22.0.0']

setup_kwargs = {
    'name': 'chlore',
    'version': '0.4.0',
    'description': 'Web utilities with a good smell',
    'long_description': None,
    'author': 'Clément "Doom" Doumergue',
    'author_email': 'clement.doumergue@etna.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from typing import Literal, Type, TypeVar

from pydantic import BaseSettings


class LoggingConfig(BaseSettings):
    format: Literal["console", "json"] = "console"


class DatabaseConfig(BaseSettings):
    url: str


T = TypeVar("T", bound=BaseSettings)


def from_env(t: Type[T], with_prefix: str = "") -> T:
    class WithPrefix(t):
        class Config:
            env_prefix = with_prefix

    return WithPrefix()
